
"""Evaluate required file example. This file is the template and you can change all the methods and functions in here except the method call segmentate. The method segmentate must receive a unique input and return an unique output. Read more indications on the function.
Made by: Edgar RP
Version: 1.1
"""
import numpy as np
import tensorflow as tf


def preprocessing(data: np.ndarray) -> np.ndarray:
    """Returns a min-max normalized data array with the pixel values in the range [0, 1]."""
    min = data.min()
    max = data.max()
    data = (data - min) / (max - min)
    resized_data = tf.image.resize(data, (224, 224))
    return resized_data


def segmentate(input_data):
    """This function is where you load and apply the post-processing of the images befores entering the model. It must return a list of numpy arrays with shape (B, H, W, S) and mantaining the order as input_data is given. B -> Batch, H -> Height, W -> Width and S -> Slices. The input_data in batch dimension is equal to 40 because are all the test cases for APIS and the output data must have the same 40 cases in the same order and mantain the original shape of input data.
    Args:
        input_data (List[np.array]): The input data dicoms without any reescaling (samples are as loaded with nibabel) where each array have the shape (H, W, S).
    Returns:
        output_data (List[np.array]): The prediction of the segmentation masks of the model as List of arrays where each array must have the format (H, W, S) and must be boolean arrays.
    """
    # Load the model.
    model = tf.keras.models.load_model("apis_baseline.h5")

    # Preprocessing.
    preprocessed_data = preprocessing(input_data)

    # For each volume, predict over the slices.
    result = []
    for ncct in preprocessed_data:
        volume = tf.expand_dims(ncct.transpose(2, 0, 1), axis=-1)
        preds = model.predict(volume, batch_size=1)  # S, H, W, 1
        mask = (preds > 0.5).astype(np.float32)
        result.append(mask[..., -1].transpose(1, 2, 0))
    output_data = np.array(result)

    return output_data
